#!/bin/bash

# Create granule-based reprojected VIIRS GeoTIFF images
# Requires GMTCO, GITCO, GDNBO, SVM03, SVM04, SVM05, SVI01, SVI02, SVI05, and SVDNB files from SNPP or NOAA-20 in the same directory

# Requires CSPP Polar2Grid v2.2.1

# Set up P2G
export POLAR2GRID_HOME=$HOME/polar2grid_v_2_2_1
source $POLAR2GRID_HOME/bin/polar2grid_env.sh

# Get VIIRS GMTCO filename
file=$1
if [ ! -r $file ]; then
  echo "File not found "$file
  exit 1
fi
echo
echo "Processing "$file

# Extract directory and file header (e.g., "npp_d20180911_t0744")
dir=$(dirname $file)
header=$(basename $file | cut -d"_" -f2-4)

# Get center lat/lon and solar zenith from geolocation file
data="/All_Data/VIIRS-MOD-GEO-TC_All/Longitude"
clon=$(h5dump -d $data -s "384,1600" -c "1,1" $file | awk 'NR==11' | awk '{print $2}')
data="/All_Data/VIIRS-MOD-GEO-TC_All/Latitude"
clat=$(h5dump -d $data -s "384,1600" -c "1,1" $file | awk 'NR==11' | awk '{print $2}')
data="/All_Data/VIIRS-MOD-GEO-TC_All/SolarZenithAngle"
czen=$(h5dump -d $data -s "384,1600" -c "1,1" $file | awk 'NR==11' | awk '{print $2}')
echo

# Define orthographic projection grid
xres=500    # meters
yres=500    # meters
echo "ortho500m, proj4, +proj=ortho +datum=WGS84 +ellps=WGS84 +lon_0=${clon} +lat_0=${clat}, None, None, ${xres}, -${yres}, None, None" > temp.grd 

# Create reprojected images for day and night
polar2grid.sh viirs_sdr gtiff -p i02 i05 hncc_dnb --grid-configs temp.grd -g ortho500m --grid-coverage=0.01 -f $dir/SV{I02,I05,DNB}_${header}*.h5 $dir/G{DNBO,ITCO}_${header}*.h5

# Create reprojected images for day only
day=$(echo "${czen} < 89.0" | bc)    # Returns 1 if czen is less than 89.0
if [ $day -eq 1 ]; then
  polar2grid.sh crefl gtiff --true-color --grid-configs temp.grd -g ortho500m -f $dir/SV{I01,M03,M04,M05}_${header}*.h5 $dir/G{MTCO,ITCO}_${header}*.h5
fi

# Cleanup
rm crefl*.log CREFL*.hdf viirs_sdr*.log *viirs_crefl*tif temp.grd
